@@ -29,7 +29,7 @@ pub struct Annotation {
 
 #[derive(Debug)]
 pub enum AnnotationKind {
-    Runnable { debug: bool, runnable: Runnable },
+    Runnable(Runnable),
     HasImpls { position: FilePosition, data: Option<Vec<NavigationTarget>> },
     HasReferences { position: FilePosition, data: Option<Vec<FileRange>> },
 }
@@ -40,14 +40,12 @@ pub struct AnnotationConfig {
     pub annotate_impls: bool,
     pub annotate_references: bool,
     pub annotate_method_references: bool,
-    pub run: bool,
-    pub debug: bool,
 }
 
 pub(crate) fn annotations(
     db: &RootDatabase,
+    config: &AnnotationConfig,
     file_id: FileId,
-    config: AnnotationConfig,
 ) -> Vec<Annotation> {
     let mut annotations = Vec::default();
 
@@ -57,24 +55,9 @@ pub(crate) fn annotations(
                 continue;
             }
 
-            let action = runnable.action();
             let range = runnable.nav.focus_or_full_range();
 
-            if config.run {
-                annotations.push(Annotation {
-                    range,
-
-                    // FIXME: This one allocates without reason if run is enabled, but debug is disabled
-                    kind: AnnotationKind::Runnable { debug: false, runnable: runnable.clone() },
-                });
-            }
-
-            if action.debugee && config.debug {
-                annotations.push(Annotation {
-                    range,
-                    kind: AnnotationKind::Runnable { debug: true, runnable },
-                });
-            }
+            annotations.push(Annotation { range, kind: AnnotationKind::Runnable(runnable) });
         }
     }
 
@@ -190,16 +173,14 @@ mod tests {
 
         let annotations: Vec<Annotation> = analysis
             .annotations(
-                file_id,
-                AnnotationConfig {
+                &AnnotationConfig {
                     binary_target: true,
                     annotate_runnables: true,
                     annotate_impls: true,
                     annotate_references: true,
                     annotate_method_references: true,
-                    run: true,
-                    debug: true,
                 },
+                file_id,
             )
             .unwrap()
             .into_iter()
@@ -225,9 +206,9 @@ fn main() {
                 [
                     Annotation {
                         range: 53..57,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -240,26 +221,7 @@ fn main() {
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 53..57,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 50..85,
-                                    focus_range: 53..57,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 6..10,
@@ -329,9 +291,9 @@ fn main() {
                 [
                     Annotation {
                         range: 17..21,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -344,26 +306,7 @@ fn main() {
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 17..21,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 14..48,
-                                    focus_range: 17..21,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 7..11,
@@ -437,9 +380,9 @@ fn main() {
                 [
                     Annotation {
                         range: 69..73,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -452,26 +395,7 @@ fn main() {
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 69..73,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 66..100,
-                                    focus_range: 69..73,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 7..11,
@@ -598,9 +522,9 @@ fn main() {}
                 [
                     Annotation {
                         range: 3..7,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -613,26 +537,7 @@ fn main() {}
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 3..7,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 0..12,
-                                    focus_range: 3..7,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 3..7,
@@ -671,9 +576,9 @@ fn main() {
                 [
                     Annotation {
                         range: 61..65,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -686,26 +591,7 @@ fn main() {
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 61..65,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 58..95,
-                                    focus_range: 61..65,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 7..11,
@@ -813,9 +699,9 @@ mod tests {
                 [
                     Annotation {
                         range: 3..7,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -828,32 +714,13 @@ mod tests {
                                 kind: Bin,
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 3..7,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 0..12,
-                                    focus_range: 3..7,
-                                    name: "main",
-                                    kind: Function,
-                                },
-                                kind: Bin,
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 18..23,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -869,35 +736,13 @@ mod tests {
                                 },
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 18..23,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 14..64,
-                                    focus_range: 18..23,
-                                    name: "tests",
-                                    kind: Module,
-                                    description: "mod tests",
-                                },
-                                kind: TestMod {
-                                    path: "tests",
-                                },
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 45..57,
-                        kind: Runnable {
-                            debug: false,
-                            runnable: Runnable {
+                        kind: Runnable(
+                            Runnable {
+                                use_name_in_title: false,
                                 nav: NavigationTarget {
                                     file_id: FileId(
                                         0,
@@ -917,33 +762,7 @@ mod tests {
                                 },
                                 cfg: None,
                             },
-                        },
-                    },
-                    Annotation {
-                        range: 45..57,
-                        kind: Runnable {
-                            debug: true,
-                            runnable: Runnable {
-                                nav: NavigationTarget {
-                                    file_id: FileId(
-                                        0,
-                                    ),
-                                    full_range: 30..62,
-                                    focus_range: 45..57,
-                                    name: "my_cool_test",
-                                    kind: Function,
-                                },
-                                kind: Test {
-                                    test_id: Path(
-                                        "tests::my_cool_test",
-                                    ),
-                                    attr: TestAttr {
-                                        ignore: false,
-                                    },
-                                },
-                                cfg: None,
-                            },
-                        },
+                        ),
                     },
                     Annotation {
                         range: 3..7,
